package be.ggj.elektrofried.io;

import gnu.io.CommPortIdentifier;
import gnu.io.SerialPort;
import gnu.io.SerialPortEvent;
import gnu.io.SerialPortEventListener;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Enumeration;

import be.ggj.elektrofried.game.ElektroFriedGame;
import be.ggj.elektrofried.sprites.SteerableElektron;

public class SerialPortTest implements SerialPortEventListener, PropertyChangeListener {
	SerialPort serialPort;
	/** The port we're normally going to use. */
	private static final String PORT_NAMES[] = { 
		"/dev/tty.usbserial-A9007UX1", // Mac OS X
		"/dev/ttyUSB0", // Linux
		"COM3", // Windows
	};
	/** Buffered input stream from the port */
	private InputStream input;
	/** The output stream to the port */
	private OutputStream output;
	/** Milliseconds to block while waiting for port open */
	private static final int TIME_OUT = 2000;
	/** Default bits per second for COM port. */
	private static final int DATA_RATE = 9600;

	private SteerableElektron steerableElektron;
	private boolean lightingLeds;

	private final static String NUMBER_SEQUENCE =  "abcdef";

	public void initialize() {
		CommPortIdentifier portId = null;
		try {
			Enumeration portEnum = CommPortIdentifier.getPortIdentifiers();

			// iterate through, looking for the port
			while (portEnum.hasMoreElements()) {
				CommPortIdentifier currPortId = (CommPortIdentifier) portEnum.nextElement();
				for (String portName : PORT_NAMES) {
					if (currPortId.getName().equals(portName)) {
						portId = currPortId;
						break;
					}
				}
			}

			if (portId == null) {
				System.out.println("Could not find COM port.");
				return;
			}

			// open serial port, and use class name for the appName.
			serialPort = (SerialPort) portId.open(this.getClass().getName(),
					TIME_OUT);

			// set port parameters
			serialPort.setSerialPortParams(DATA_RATE,
					SerialPort.DATABITS_8,
					SerialPort.STOPBITS_1,
					SerialPort.PARITY_NONE);

			// open the streams
			input = serialPort.getInputStream();
			output = serialPort.getOutputStream();

			// add event listeners
			serialPort.addEventListener(this);
			serialPort.notifyOnDataAvailable(true);
		} catch (Throwable e) {
			System.err.println(e.toString());
		}
	}

	/**
	 * This should be called when you stop using the port.
	 * This will prevent port locking on platforms like Linux.
	 */
	public synchronized void close() {
		if (serialPort != null) {
			serialPort.removeEventListener();
			serialPort.close();
		}
		steerableElektron = null;
	}



	/**
	 * Handle an event on the serial port. Read the data and print it.
	 */
	public synchronized void serialEvent(SerialPortEvent oEvent) {
		if (oEvent.getEventType() == SerialPortEvent.DATA_AVAILABLE) {
			try {
				int available = input.available();
				byte chunk[] = new byte[available];
				input.read(chunk, 0, available);
				// Displayed results are codepage dependent
				String result = new String(chunk);
				if (result.contains("3")) {
					steerableElektron.changeTargetRadius(8);
					System.out.println("got byte, radius increased .. ");
				} else if (result.contains("2")) {
					steerableElektron.changeTargetRadius(-8);
					System.out.println("got byte, radius decreased .. ");
				}
				System.out.println(result);

			} catch (Exception e) {
				System.err.println(e.toString());
			}
		}
		// Ignore all the other eventTypes, but you should consider the other ones.
	}

	public void shock(int strength) {
		try {
			if (output != null) {
				output.write(new String(strength + "\r\n").getBytes());
				System.out.println("SHOCK LEVEL " + strength);
			} else {
				System.out.println("no serial device attached..");
			}
		} catch (IOException e1) {
			e1.printStackTrace();
		}
	}

	public boolean isLightingLeds() {
		return lightingLeds;
	}

	public void setLightingLeds(boolean lightingLeds) {
		this.lightingLeds = lightingLeds;
	}

	public void lightLeds(int level) {
		try {
			if (output != null) {
				for (int i = 0; i < 3; i++) {
					output.write( "agbhcidjekdjcibh\r\n".getBytes());
				}
				output.write(new String(NUMBER_SEQUENCE.substring(0, level) +  "\r\n").getBytes());
			} else {
				System.out.println("no serial device attached..");
			}
		} catch (Exception e1) {
			e1.printStackTrace();
		}
	}

	@Override
	public void propertyChange(PropertyChangeEvent evt) {
		if (evt.getPropertyName().equals(ElektroFriedGame.STEERABLE_ELEKTRON)) {
			this.steerableElektron = (SteerableElektron) evt.getNewValue();
		} else if (evt.getPropertyName().equals(ElektroFriedGame.LEVEL)) {
			lightLeds((Integer) evt.getNewValue());
		}
	}

}

